from wisup_e2m.configs.parsers.doc_parser_config import DocParserConfig
from wisup_e2m.configs.parsers.docx_parser_config import DocxParserConfig
from wisup_e2m.configs.parsers.pdf_parser_config import PdfParserConfig
from wisup_e2m.configs.parsers.epub_parser_config import EpubParserConfig
from wisup_e2m.configs.parsers.ppt_parser_config import PptParserConfig
from wisup_e2m.configs.parsers.pptx_parser_config import PptxParserConfig
from wisup_e2m.configs.parsers.url_parser_config import UrlParserConfig
from wisup_e2m.configs.parsers.html_parser_config import HtmlParserConfig
from wisup_e2m.configs.parsers.voice_parser_config import VoiceParserConfig

__all__ = [
    "DocParserConfig",
    "DocxParserConfig",
    "PdfParserConfig",
    "EpubParserConfig",
    "PptParserConfig",
    "PptxParserConfig",
    "UrlParserConfig",
    "HtmlParserConfig",
    "VoiceParserConfig",
]
